open Lexing  (* distributed with OCaml *)
open Lexer
open Parser
open Pretty
open Ast


let dotty = ref false 
let parse = ref true
let ast2ir = ref true
let file = ref ""

let speclist = [
  "-dot", Arg.Set dotty, "\tSet dotty output" ;
  "-noparse", Arg.Clear parse, "\tDisable parsing (implies -notrans)";
  "-notrans", Arg.Clear ast2ir, "\tDisable translation AST -> IR"
  ]

let anon_fun s =
  if !file = ""
  then file := s
  else failwith "Too many files"

let usage_msg =
  "\nUsage: " ^ Sys.argv.(0) ^ " [options] file.pas\n"


let main =
  Arg.parse speclist anon_fun usage_msg ;

  if !file = "" then
    failwith ("Try " ^ Sys.argv.(0) ^ " -help") ;

  if !dotty && not !parse then
    failwith "You cannot choose -dot AND -noparse" ;


  let input = open_in_bin !file in
  let lexbuf = Lexing.from_channel input
  and p =
    {
     pos_fname = !file;
     pos_lnum = 1;
     pos_bol = 0;
     pos_cnum = 0;
    } in
  lexbuf.lex_curr_p <- p ;

  if not !parse then
    while (
      let t = token lexbuf in
      print_token t ;
      t <> EOF )
    do
      flush stdout
    done
  else 
    begin
      let ast = Parser.main Lexer.token lexbuf in

      if not !ast2ir then begin
        if !dotty then Ast.to_graph "out.dot" ast
        else
          print_endline (Ast.to_string ast)

      end else begin

        (* Translate AST to IR *)
        Semantic.trans_ast ast ;

        (* Get the fragments *)
        let frag_list = Translate.get_result () in

        (* Display them *)
        List.iter (
          fun f -> match f with
          | Frame.String (lab,s) ->
              Printf.printf "String Label %s: '%s'\n" lab s
          | Frame.Fun (f,stm) ->
              Printf.printf "Frame of %s:\n" (Frame.name f);
              if !dotty then (
                let s = Frame.name f ^ ".dot" in
                Printf.printf "\tdotty in %s\n" s ;
                Ir.to_graph s stm
              )
              else
                Ir.print stm
          )
          frag_list
      end
    end
