open Temporary
open Ast
open Types
open Translate

type venv = Env.enventry Symbol.table
type tenv = Types.ty Symbol.table
type var = Ast.var
type exp = Ast.exp
type dec = Ast.dec
type ty = Ast.ty_ty
type ast = Ast.ast
type frame = Frame.frame


(** a counter used to name the labels *)
let num = ref 0
let get_num () = incr num ; !num


(** this value is used to distinguish the main procedure from others *)
let is_main = ref false

(** for error messages *)
let position_of_var = function
  | Var_or_Const (_,p)
  | Field_access (_,_,p)
  | Array_access (_,_,p) -> p
  | Fun_access (Call_exp c) -> c.call_pos
  | _ -> failwith "other cases not expected in Semantic.pos_of_var"

(** verify a type is what it should be *)
let verify_type to_verify should_be position msg =
  if not (equal to_verify should_be)
  then raise (Fatal_error (
	      Error_handler.error_msg_pos position ^
	      msg ^ " has type " ^
	      to_string to_verify ^
	      " but should be " ^
	      to_string should_be))


(**  remove_sameas: Types.ty Symbol.table -> Ast.ty_dec -> Types.ty
 *
 * Return the type of [t_dec] where every SAMEAS has been replaced by the
 * type of the corresponding symbol in [tenv].
 * 
 * *)
let remove_sameas tenv t_dec =

  ...


  (** [trans_var frame venv tenv var] translates the variable [var] into an expression which is
   * an access to this variable in the frame or in a register.
   *
   * If the variable is partial access (e.g. t[i] if t is a two-dimensionnal array)
   * the expression returned must be the address in the frame and not the value at this address.
   * So there is a problem which is to know if a MEM node must be added or not.
   * 
   * If [var] is a constant or a simple variable access (i.e. without Array_access or Record_access)
   * a MEM node is never added.
   *
   * If [var] is an array or a record access, there are two cases :
   *   1) the value accessed is the deepest value accessible according to the
   *      type of the variable (e.g. x has type (RECORD([tab,ARRAY(INT,_,_,_)],_))
   *      and the variable access [var] is x.tab[3]).
   *      In this case a MEM node is added.
   *   2) else there is no MEM node to add.
   *      (e.g. suppose x has the same type as in the first case and
   *      the variable access [var] is x.tab)
   * 
   * In order to know if a MEM node is added or not, a constructor PARREC will be added in front of
   * the type of an array or a record variable.
   * Then the PARREC constructor follows the analysis of the variable access.
   * 
   * For example if the variable is of type [PARREC (ARRAY t)] 
   * where t is a basic type (INT, STRING, BOOL, POINTER...), a MEM node is added.
   * 
   * This constructor is useful since arrays or records have a linear memory
   * representation. An array of arrays is not an array of pointers on arrays.
   * 
   * For example, if :
   * 
   *   x : array of arrays of integer ;
   * 
   *   x[1] is an array. Its access is : Array_access ( Var_or_Const x, 1).
   * 
   *   [trans_var (Var_or_Const x)] returns the record [{exptsl = e ; expty = PARREC of ARRAY of ARRAY of integers}].
   *   Therefore [trans_var (Array_access ( Var_or_Const x, 1))] returns the record
   *   [{exptsl = e' ; expty = PARREC of ARRAYS of integers ;}]
   * 
 *)


let rec trans_var frame venv tenv var =

  ...


(** translate a sequence into an IR expression *)
and trans_seq frame venv tenv l =


(**
  translate an ast expression into an IR tree expression.
  All results are of type [Ir.stm] even if it is an IR expression.
  Consequently we add Ir.EXP node or IR.SEQ node which will be
  remove by the Canon module.

  val trans_exp :
  Frame.frame ->
  Env.enventry Symbol.table ->
  Types.ty Symbol.table ->
  exp ->
  Translate.expty *)
and trans_exp frame venv tenv e =


(** [destruct_test pos exp] convert an expression into a relop operation.
  For example, BINOP (AND,A,B) is converted into RELOP (EQ,1,BINOP (AND,A,B))
*)
  let destruct_test pos = function
    ...
  in
    ...

and trans_dec frame venv tenv dec =

  ...


and trans_ty frame venv tenv =
  
  ...
  

let trans_ast ast =

  (* label of the main function, assembly-dependant *) in
  let label = ... 

  (* Frame for the main *)
  let frame = Frame.new_frame label [] 0 in

  (* Variable and type environments *)
  let venv, tenv = Symbol.create () , Symbol.create () in
  in

  (* predefined constants and predefined types
  are added to symbol tables *)
  ignore( Predefined.add venv tenv ) ;

  is_main := true ;

  trans_dec frame venv tenv
  (
    Proc_dec
    [ Ast.ast_to_proc ast ]
  )
